#   Copyright 2012-2013 Eric Ptak - trouch.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

from webiopi.utils.types import toint, signInteger
from webiopi.devices.i2c import I2C
from webiopi.devices.sensor import Acceleration



class ADXL345(I2C, Acceleration):
	def __init__(self, slave=0x53):
		I2C.__init__(self, toint(slave))
		self.writeRegister(0x31,0x0B) 
		self.writeRegister(0x1E,0x00)#X轴误差补偿; (15.6mg/LSB)
		self.writeRegister(0x1F,0x00)#Y轴误差补偿; (15.6mg/LSB)
		self.writeRegister(0x20,0x00)#Z轴误差补偿; (15.6mg/LSB)
		self.writeRegister(0x21,0x00)#敲击延时0:禁用;(1.25ms/LSB)
		self.writeRegister(0x22,0x00)#检测第一次敲击后的延时0:禁用;(1.25ms/LSB)
		self.writeRegister(0x23,0x00)#敲击窗口0:禁用;(1.25ms/LSB)
		self.writeRegister(0x24,0x01)#保存检测活动阀值;(62.5mg/LSB)
		self.writeRegister(0x25,0x01)#保存检测静止阀值;(62.5mg/LSB)
		self.writeRegister(0x26,0x2B)#检测活动时间阀值;(1s/LSB)
		self.writeRegister(0x27,0x00)#
		self.writeRegister(0x28,0x09)#自由落体检测推荐阀值;(62.5mg/LSB)
		self.writeRegister(0x29,0xFF)#自由落体检测时间阀值,设置为最大时间;(5ms/LSB)
		self.writeRegister(0x2A,0x80)#禁止敲击检测
		self.writeRegister(0x2C,0x08)#200hz 输出0A是100hz输出
		self.writeRegister(0x2D,0x08)#开启Link,测量功能;关闭自动休眠,休眠,唤醒功能
		self.writeRegister(0x2E,0x80)#所有均关闭
		self.writeRegister(0x2F,0x00)#中断功能设定,不使用中断
		self.writeRegister(0x38,0x9F)#FIFO模式设定,bypass模式,31级样本缓冲
	
	def __str__(self):
		return "ADXL345(slave=0x%02X)" % self.slave
		
	def __getAcceX__(self):
		d0 = self.readRegisters(0x32 , 2)
		count = ((d0[0] << 4) | (d0[1] >> 4)) & 0xFFFF
		return signInteger(count, 13) * 0.000244140625
	def __getAcceY__(self):
		d1 = self.readRegisters(0x34 , 2)
		count = ((d1[0] << 4) | (d1[1] >> 4)) & 0xFFFF
		return signInteger(count, 13) * 0.000244140625
	def __getAcceZ__(self):
		d2 = self.readRegisters(0x36 , 2)
		count = ((d2[0] << 4) | (d2[1] >> 4)) & 0xFFFF
		return signInteger(count, 13) * 0.000244140625
